// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
#include<iostream>
#include "param.h"
#include<vector>
#include "qdir.h"
#include "qstring.h"
#include "qfileinfo.h"
#include "Dal.h"
#include"errstr.h"
#include <cmath>
#include <cstring>
/***************************************************************
*
*
*
* 
****************************************************************/
void modifyFileName(string &file);

extern "C" void modifydirectorylist_();
extern "C" void modifyimagelist_();
extern "C" void modifysrclist_();



extern "C" void loadimagefiles_( int *directoryNumber )
{


   
      
    string filter = stringParameter("filter");
   

    string directory = stringParameter("directorylist", *directoryNumber);


    //cout << "L37 - directory=" << directory << " directorynumber=" << *directoryNumber << endl;   

    int k = directory.length() - 1;
    if ( !( (directory.at(k) == '/') || (directory.at(k) == '\\')) )
    {
        if ( directory.find('/') !=string::npos)
            directory.append("/");
        else
            directory.append("\\");
    }
     
      

    QDir thisDir( directory.c_str() );
    if ( !thisDir.isReadable() ) 
    {
        errstr << "Could not open directory " << directory << Sas::warning("BadDirectory");
        return;

    }


    errstr << "Loading image files from " << directory << Sas::message(Msg::App, Msg::Verbose);
    printf("L60 loadimagefiles.cc \n");


    int binning =  intParameter("binning");

    stringListParameter("imagesets").clear();
   
    stringListParameter("srclistsets").clear();

    vector<string> emptyList(0);
 
    setParameter("imagesets", emptyList);  

    setParameter("srclistsets", emptyList);  


    vector<string> imageList;    

    //cout << "L77 thisDir.entryInfoList=" << thisDir.entryInfoList() << endl;   
    const QFileInfoList * files = thisDir.entryInfoList();
    if ( files ) 
    {
      //cout << "L81 " << endl;
	
        QFileInfoListIterator it( *files );
	//cout << "L82 it.current()=" << it.current() << endl;
        QFileInfo * fi;
        while( (fi = it.current()) != 0 ) 
        {
            ++it;
            QString file = directory.c_str() + fi->fileName();
	    //cout << "L89 file=" << file << endl;

            /************************************************************
	    *
            *
            *
            *
            ************************************************************/
            if ( ( file.contains("P0")      && 
                   file.contains("IMAGE_")  &&
                   ( file.contains("FIT") || file.contains("FTZ") ) )   ||
             
                   ( file.contains("P0")      && 
                     file.contains("FIMAG_")  &&  
		     ( file.contains("FIT") || file.contains("FTZ")) ))
             
            {
	      /*
	      cout << "L107 file=" << file << endl;
	      */
	     

	        if( !QFile::exists(file) )   
		{
		  continue; 

		}    
               
                // Open the file
                DataSet *dataSet = dataSetServer->open((const char *)file, Dal::Read);
                if (dataSet)
	        {
		    string filter1 =  dataSet->attribute("FILTER")->asString();
                    int binX =  (int)ceil(pow(2.0, dataSet->attribute("BINAX1")->asInt()));
                    bool binBpe =  dataSet->attribute("BINBPE")->asBool();
                    if(binBpe) {
		      binX *= 2;
                    }
		    /*
                    cout << "L128 filter=" << filter << " filter1=" << filter1 
			 << " binning=" << binning << " binX="<< binX << endl;
		    */
		    if(filter == filter1 && binning == binX )
		    {
                        if (file.contains(".\\"))
                        {
                            file.remove(".\\"); 
                        }
                        else if(file.contains("./"))
                        {
                            file.remove("./"); 
                        }

		        imageList.push_back( (const char *)file );
                        errstr << "adding file " << file << " to the list to be processed" << Sas::message(Msg::App, Msg::Verbose);
			/*
			cout << "L142 adding file=" << file << " to the list " << endl;
			*/
			//			cout << "adding file " << file << " to the list to be processed\n";
 
                    } 

                    dataSetServer->close(dataSet); // Finished with the file

                }
            }
        }
       
    }
    //   cout << "Line 136 imagelist.size()=" << imageList.size() << endl;

    if(imageList.size() > 0)
    {
        setParameter("imagesets", imageList);  
        errstr << "Final number of files to be processed=" << imageList.size() << Sas::message(Msg::App, Msg::Verbose);
 
	//    cout << "loadimagefiles line 142- the images files\n";
	// for(int i = 0; i < parameterCount("imagesets"); i++)
	//  cout << i << ") " << stringParameter("imagesets", i);


    }
    else
    {
      errstr << "No images files for filter " << filter << " have been found" << Sas::warning("NoImagesToProcess");

      // if( parameterCount("imagesets") > 0)
      // {
      //	cout << " number of image files=" << parameterCount("imagesets") << endl;   

      //	cout << "Bug at line 155\n";
      //  exit(0);

      //      }
 

    }
  

}
/**********************************************************************************
*
*
*
*
**********************************************************************************/



void modifyFileName(string &file)
{
 

    
    while  ( file.find("//")  !=string::npos)
    {
        file.replace( file.find("//"), 2, "/");
    }

    while  ( file.find("\\")  !=string::npos)
    {
        file.replace( file.find("\\"), 2, "\\");
    }

        

}
/**********************************************************
*
*
*
**********************************************************/

extern "C" void modifydirectorylist_()

{

    vector<string> list = stringListParameter("directorylist");
    

    for(unsigned i = 0; i < list.size(); i++)
    {
        modifyFileName( list[i] ); 
        int length = list[i].length();
        if( length > 2) 
        { 
            if( list[i].find("/")  !=string::npos)
	    {
              if( list[i].at(length - 1) != '/')
	      { 
	          list[i].append("/");
              }
            }
            else
	    {

                if( list[i].find("\\")  !=string::npos)
	        {
                    if( list[i].at(length - 1) != '\\')
	            {
	                list[i].append("\\");
                    }
                }
            }
     
	}

    }    


    setParameter("directorylist", list);  
   

}
/**********************************************************
*
*
*
**********************************************************/

extern "C" void modifyimagelist_()

{

    vector<string> list = stringListParameter("imagesets");
    

    for(unsigned i = 0; i < list.size(); i++)
    {
        modifyFileName( list[i] ); 


    }    

    setParameter("imagesets", list);  
   

}

/**********************************************************
*
*
*
**********************************************************/

extern "C" void modifysrclist_()

{

    vector<string> list = stringListParameter("srclistsets");
    

    for(unsigned i = 0; i < list.size(); i++)
    {
        modifyFileName( list[i] ); 


    }    

    setParameter("srclistsets", list);  
   

}
